from pyLandau import landau
# Check the integral of PDF ~ 1 with scipy numerical integration
mu, eta, sigma, A = 10, 1, 3, 1
from scipy import integrate
y, err = integrate.quad(landau.get_landau_pdf, 0, 10000, args=(mu, eta))
print 'Integral of Landau PDF:', y
y, err = integrate.quad(landau.get_gauss_pdf, 0, 10000, args=(mu, sigma))
print 'Integral of Gauss PDF:', y
y, err = integrate.quad(landau.get_langau_pdf, -10000, 10000, args=(mu, eta, sigma))
print 'Integral of Landau + Gauss (Langau) PDF:', y


from scipy.optimize import curve_fit
import matplotlib.pyplot as plt
import numpy as np
from pyLandau import landau

x = np.arange(0, 100, 0.5)
mu, eta, sigma, A = 10, 1, 3, 1
y = landau.langau(x, mu, eta, sigma, A) + np.random.normal(0, 0.05, 200)

coeff, pcov = curve_fit(landau.langau, x, y, p0=(mu, eta, sigma, A))
plt.plot(x, y, "o")
plt.plot(x, landau.langau(x, *coeff), "-")
plt.show()



''' Script to check pyLandau.
'''
import unittest
from pyLandau import landau


def approx_Equal(x, y, tolerance=0.001):  # check similarity of values within an allowed tolerance
    return abs(x - y) <= 0.5 * tolerance * (x + y)


def drange(start, stop, step):  # range with floats
    r = start
    while r < stop:
        yield r
        r += step


def F(function, x, args=()):
    return (function(x_i, *args) for x_i in x)


def simple_integrate(function, a, b, args=(), dx=1.):  # integration with sum, substitutes scipy.integrate.quad
    x = drange(a, b, dx)
    y = F(function, x, args)
    return sum(y) * dx, x


try:  # use scipy integrate function if available, otherwise self defined simple_integrate
    from scipy.integrate import quad as integrate
except ImportError:
    integrate = simple_integrate


class Test(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        pass

    @classmethod
    def tearDownClass(cls):  # remove created files
        pass

    def test_check_pdf_integral(self):  # a pdf integral has to be 1
        mu, eta, sigma = 10, 1, 3
        result, _ = integrate(landau.get_landau_pdf, 0, 10000, args=(mu, eta))
        self.assertTrue(approx_Equal(result, 1))
        result, _ = integrate(landau.get_gauss_pdf, 0, 10000, args=(mu, sigma))
        self.assertTrue(approx_Equal(result, 1))
        result, _ = integrate(landau.get_langau_pdf, -10000, 10000, args=(mu, eta, sigma))
        self.assertTrue(approx_Equal(result, 1))


if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(Test)
    unittest.TextTestRunner(verbosity=2).run(suite)



import numpy as np
from pyLandau import landau

# Plot the functions using matplotlib
import matplotlib.pyplot as plt
x = np.arange(0, 100, 0.01)

for A, eta, mu in ((1, 1, 10), (1, 2, 30), (0.5, 5, 50)):
    plt.plot(x, landau.landau(x, mu, eta, A), label='A=%d, eta=%d, mu=%d' % (A, eta, mu))
plt.legend(loc=0)
plt.show()

# Plot the Landau Gauss convolution and cross check with convolution with scipy
from scipy.ndimage.filters import gaussian_filter1d
mu, eta, sigma, A = 10, 1, 3, 1
x = np.arange(0, 50, 0.01)
y = landau.landau(x, mu=mu, eta=eta, A=A)
y_gconv = gaussian_filter1d(y, sigma=sigma / 0.01)
y_gconv_2 = landau.langau(x, mu, eta, sigma, A)
plt.plot(x, y, label='Landau')
plt.plot(x, y_gconv_2, label='Langau')
plt.plot(x, y_gconv / np.amax(y_gconv), '--', label='Langau Scipy')
plt.legend(loc=0)
plt.show()


# distutils: language = c++
# cython: boundscheck=False
# cython: wraparound=False

import numpy as np
cimport numpy as cnp
from libcpp.vector cimport vector
cnp.import_array()

cdef extern from "numpy/arrayobject.h":
    void PyArray_ENABLEFLAGS(cnp.ndarray arr, int flags)

cdef extern from "landau_src.cpp":
    double* getLandauPDFData(double*& data, const unsigned int& size, const double& mu, const double& eta) except +
    double* getLangauPDFData(double*& data, const unsigned int& size, const double& mu, const double& eta, const double& sigma) except +
    double landauPDF(const double& x, const double& xi, const double& x0) except +
    double landauGaussPDF(const double& x, const double& mu, const double& eta, const double& sigma) except +
    double gaussPDF(const double& x, const double& mu, const double& sigma)


cdef data_to_numpy_array_double(cnp.double_t * ptr, cnp.npy_intp N):
    cdef cnp.ndarray[cnp.double_t, ndim = 1] arr = cnp.PyArray_SimpleNewFromData(1, < cnp.npy_intp * > & N, cnp.NPY_DOUBLE, < cnp.double_t * > ptr)
    PyArray_ENABLEFLAGS(arr, cnp.NPY_OWNDATA)
    return arr

cdef cnp.double_t* result = NULL


def get_landau_pdf(value, mu=0, eta=1):
    return landauPDF(<const double&> value, <const double&> mu, <const double&> eta)


def get_gauss_pdf(value, mu=0, sigma=1):
    return gaussPDF(<const double&> value, <const double&> mu, <const double&> sigma)


def get_langau_pdf(value, mu=0, eta=1, sigma=1):
    return landauGaussPDF(<const double&> value, <const double&> mu, <const double&> eta, <const double&> sigma)


def get_landau(value, mu=0, eta=1, A=1):
    landau_pdf = get_landau_pdf(value - 0.22278298, mu, eta)  # correct value that MPV is mu
    return landau_pdf / np.amax(landau_pdf) * A


def get_langau(value, mu=0, eta=1, sigma=1, A=1):
    langau_pdf = get_langau_pdf(value, mu, eta, sigma)  # shift correction done in function
    return langau_pdf / np.amax(langau_pdf) * A


def landau_pdf(cnp.ndarray[cnp.double_t, ndim=1] array, mu=0, eta=1):
    result = getLandauPDFData(< double*& > array.data, < const unsigned int&> array.shape[0], < const double&> mu, < const double&> eta)
    return data_to_numpy_array_double(result, array.shape[0])


def langau_pdf(cnp.ndarray[cnp.double_t, ndim=1] array, mu=0, eta=1, sigma=1):
    result = getLangauPDFData(< double*& > array.data, < const unsigned int&> array.shape[0], < const double&> mu, < const double&> eta, < const double&> sigma)
    if result != NULL:
        return data_to_numpy_array_double(result, array.shape[0])


def landau(cnp.ndarray[cnp.double_t, ndim=1] array, mu=0, eta=1, A=1):
    landau = landau_pdf(array - 0.22278298, mu, eta)  # mu is defined as the maximum (MPV), so correct function y-offset here
    return (landau / np.amax(landau) * A)


def langau(cnp.ndarray[cnp.double_t, ndim=1] array, mu=0, eta=1, sigma=1, A=1):
    langau = langau_pdf(array, mu, eta, sigma)  # mu is defined as the maximum (MPV), so correct function y-offset here
    return (langau / np.amax(langau) * A)



#!/usr/bin/env python
from setuptools import setup, find_packages, Extension  # This setup relies on setuptools since distutils is insufficient and badly hacked code
import numpy as np

# Check if cython exists, then use it. Otherwise compile already cythonized cpp file
have_cython = False
try:
    from Cython.Build import cythonize
    have_cython = True
except ImportError:
    pass

if have_cython:
    cpp_extension = cythonize(Extension('pyLandau.landau', ['pyLandau/cpp/landau.pyx']))
else:
    cpp_extension = [Extension('pyLandau.landau',
                               sources=['pyLandau/cpp/landau.cpp'],
                               language="c++")]

version = '1.1.0'
author = 'David-Leon Pohl'
author_email = 'pohl@physik.uni-bonn.de'

install_requires = ['cython', 'numpy']

setup(
    name='pyLandau',
    version=version,
    description='A Landau PDF definition to be used in Python.',
    url='https://github.com/SiLab-Bonn/pyLandau',
    license='GNU LESSER GENERAL PUBLIC LICENSE Version 2.1',
    long_description='The Landau propability density function is defined in C++ and made available to python via cython. Also a fast Gaus+Landau convolution is available. The interface accepts numpy arrays.',
    author=author,
    maintainer=author,
    author_email=author_email,
    maintainer_email=author_email,
    install_requires=install_requires,
    packages=find_packages(),
    include_package_data=True,  # accept all data files and directories matched by MANIFEST.in or found in source control
    package_data={'': ['README.*', 'VERSION'], 'docs': ['*'], 'examples': ['*']},
    ext_modules=cpp_extension,
    include_dirs=[np.get_include()],
    keywords=['Landau', 'Langau', 'PDF'],
    platforms='any'
)
